@@ -2,7 +2,7 @@
 /**
  * PHPUnit
  *
- * Copyright (c) 2002-2008, Sebastian Bergmann <sb@sebastian-bergmann.de>.
+ * Copyright (c) 2002-2009, Sebastian Bergmann <sb@sebastian-bergmann.de>.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,15 +37,22 @@
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    SVN: $Id: PhptTestCase.php 2232 2008-01-22 07:47:11Z sb $
+ * @version    SVN: $Id: PhptTestCase.php 4528 2009-01-21 14:16:59Z sb $
  * @link       http://www.phpunit.de/
  * @since      File available since Release 3.1.4
  */
 
 if (PHPUnit_Util_Filesystem::fileExistsInIncludePath('PEAR/RunTest.php')) {
+    $currentErrorReporting = error_reporting(E_ERROR | E_WARNING | E_PARSE);
+    PHPUnit_Util_Filesystem::collectStart();
     require_once 'PEAR/RunTest.php';
+    error_reporting($currentErrorReporting);
+
+    foreach (PHPUnit_Util_Filesystem::collectEnd() as $blacklistedFile) {
+        PHPUnit_Util_Filter::addFileToFilter($blacklistedFile, 'PHPUNIT');
+    }
 }
 
 require_once 'PHPUnit/Framework.php';
@@ -60,31 +67,33 @@ PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    Release: 3.2.16
+ * @version    Release: 3.3.16
  * @link       http://www.phpunit.de/
  * @since      Class available since Release 3.1.4
  */
-class PHPUnit_Extensions_PhptTestCase implements PHPUnit_Framework_Test
+class PHPUnit_Extensions_PhptTestCase implements PHPUnit_Framework_Test, PHPUnit_Framework_SelfDescribing
 {
     /**
      * The filename of the .phpt file.
      *
      * @var    string
-     * @access protected
      */
     protected $filename;
 
+    /**
+     * Options for PEAR_RunTest.
+     *
+     * @var    array
+     */
     protected $options = array();
 
     /**
      * Constructs a test case with the given filename.
      *
      * @param  string $filename
-     * @param  array  $options Array with ini settings for the php instance run,
-     *                         key being the name if the setting, value the ini value.
-     * @access public
+     * @param  array  $options
      */
     public function __construct($filename, $options = array())
     {
@@ -113,7 +122,6 @@ class PHPUnit_Extensions_PhptTestCase implements PHPUnit_Framework_Test
      * Counts the number of test cases executed by run(TestResult result).
      *
      * @return integer
-     * @access public
      */
     public function count()
     {
@@ -124,10 +132,8 @@ class PHPUnit_Extensions_PhptTestCase implements PHPUnit_Framework_Test
      * Runs a test and collects its result in a TestResult instance.
      *
      * @param  PHPUnit_Framework_TestResult $result
-     * @param  array $options Array with ini settings for the php instance run,
-     *                        key being the name if the setting, value the ini value.
+     * @param  array                        $options
      * @return PHPUnit_Framework_TestResult
-     * @access public
      */
     public function run(PHPUnit_Framework_TestResult $result = NULL, $options = array())
     {
@@ -135,6 +141,14 @@ class PHPUnit_Extensions_PhptTestCase implements PHPUnit_Framework_Test
             throw new RuntimeException('Class PEAR_RunTest not found.');
         }
 
+        if (isset($GLOBALS['_PEAR_destructor_object_list']) &&
+            is_array($GLOBALS['_PEAR_destructor_object_list']) &&
+            !empty($GLOBALS['_PEAR_destructor_object_list'])) {
+            $pearDestructorObjectListCount = count($GLOBALS['_PEAR_destructor_object_list']);
+        } else {
+            $pearDestructorObjectListCount = 0;
+        }
+
         if ($result === NULL) {
             $result = new PHPUnit_Framework_TestResult;
         }
@@ -143,19 +157,23 @@ class PHPUnit_Extensions_PhptTestCase implements PHPUnit_Framework_Test
             throw new InvalidArgumentException;
         }
 
-        $options = array_merge($options, $this->options);
-
         $coverage = $result->getCollectCodeCoverageInformation();
+        $options  = array_merge($options, $this->options);
 
-        if ($coverage) {
-            $options = array('coverage' => TRUE);
-        } else {
-            $options = array();
+        if (!isset($options['include_path'])) {
+            $options['include_path'] = get_include_path();
         }
 
-        $runner = new PEAR_RunTest(new PHPUnit_Extensions_PhptTestCase_Logger, $options);
+        if ($coverage) {
+            $options['coverage'] = TRUE;
+        } else {
+            $options['coverage'] = FALSE;
+        }
 
-        if ($coverage){
+        $currentErrorReporting = error_reporting(E_ERROR | E_WARNING | E_PARSE);
+        $runner                = new PEAR_RunTest(new PHPUnit_Extensions_PhptTestCase_Logger, $options);
+
+        if ($coverage) {
             $runner->xdebug_loaded = TRUE;
         } else {
             $runner->xdebug_loaded = FALSE;
@@ -164,9 +182,9 @@ class PHPUnit_Extensions_PhptTestCase implements PHPUnit_Framework_Test
         $result->startTest($this);
 
         PHPUnit_Util_Timer::start();
-        $buffer = $runner->run($this->filename, $options);
-        $time = PHPUnit_Util_Timer::stop();
-
+        $buffer       = $runner->run($this->filename, $options);
+        $time         = PHPUnit_Util_Timer::stop();
+        error_reporting($currentErrorReporting);
         $base         = basename($this->filename);
         $path         = dirname($this->filename);
         $coverageFile = $path . DIRECTORY_SEPARATOR . str_replace('.phpt', '.xdebug', $base);
@@ -177,11 +195,11 @@ class PHPUnit_Extensions_PhptTestCase implements PHPUnit_Framework_Test
         $phpFile      = $path . DIRECTORY_SEPARATOR . str_replace('.phpt', '.php', $base);
 
         if (is_object($buffer) && $buffer instanceof PEAR_Error) {
-            $result->addError( 
-              $this, 
+            $result->addError(
+              $this,
               new RuntimeException($buffer->getMessage()),
-              $time 
-            ); 
+              $time
+            );
         }
 
         else if ($buffer == 'SKIPPED') {
@@ -193,9 +211,7 @@ class PHPUnit_Extensions_PhptTestCase implements PHPUnit_Framework_Test
               $this,
               PHPUnit_Framework_ComparisonFailure::diffEqual(
                 file_get_contents($expFile),
-                file_get_contents($outFile),
-                FALSE,
-                $this->getName()
+                file_get_contents($outFile)
               ),
               $time
             );
@@ -207,21 +223,28 @@ class PHPUnit_Extensions_PhptTestCase implements PHPUnit_Framework_Test
             }
         }
 
-        if ($coverage) {
+        if ($coverage && file_exists($coverageFile)) {
             eval('$coverageData = ' . file_get_contents($coverageFile) . ';');
             unset($coverageData[$phpFile]);
 
-            $codeCoverageInformation = array(
-              'test'  => $this,
-              'files' => $coverageData
-            );
-
-            $result->appendCodeCoverageInformation($this, $codeCoverageInformation);
+            $result->appendCodeCoverageInformation($this, $coverageData);
             unlink($coverageFile);
         }
 
         $result->endTest($this, $time);
 
+        // Do not invoke PEAR's destructor mechanism for PHP 4
+        // as it raises an E_STRICT.
+        if ($pearDestructorObjectListCount == 0) {
+            unset($GLOBALS['_PEAR_destructor_object_list']);
+        } else {
+            $count = count($GLOBALS['_PEAR_destructor_object_list']) - $pearDestructorObjectListCount;
+
+            for ($i = 0; $i < $count; $i++) {
+                array_pop($GLOBALS['_PEAR_destructor_object_list']);
+            }
+        }
+
         return $result;
     }
 
@@ -229,7 +252,6 @@ class PHPUnit_Extensions_PhptTestCase implements PHPUnit_Framework_Test
      * Returns the name of the test case.
      *
      * @return string
-     * @access public
      */
     public function getName()
     {
@@ -240,7 +262,6 @@ class PHPUnit_Extensions_PhptTestCase implements PHPUnit_Framework_Test
      * Returns a string representation of the test case.
      *
      * @return string
-     * @access public
      */
     public function toString()
     {
