@@ -2,7 +2,7 @@
 /**
  * PHPUnit
  *
- * Copyright (c) 2002-2008, Sebastian Bergmann <sb@sebastian-bergmann.de>.
+ * Copyright (c) 2002-2009, Sebastian Bergmann <sb@sebastian-bergmann.de>.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,9 +38,9 @@
  * @package    PHPUnit
  * @author     Jan Borsodi <jb@ez.no>
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    SVN: $Id: InvocationMocker.php 1985 2007-12-26 18:11:55Z sb $
+ * @version    SVN: $Id: InvocationMocker.php 4404 2008-12-31 09:27:18Z sb $
  * @link       http://www.phpunit.de/
  * @since      File available since Release 3.0.0
  */
@@ -67,9 +67,9 @@ PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');
  * @package    PHPUnit
  * @author     Jan Borsodi <jb@ez.no>
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    Release: 3.2.16
+ * @version    Release: 3.3.16
  * @link       http://www.phpunit.de/
  * @since      Class available since Release 3.0.0
  */
@@ -111,6 +111,7 @@ class PHPUnit_Framework_MockObject_InvocationMocker implements PHPUnit_Framework
 
     public function invoke(PHPUnit_Framework_MockObject_Invocation $invocation)
     {
+        $exception      = NULL;
         $hasReturnValue = FALSE;
 
         if (strtolower($invocation->methodName) == '__tostring') {
@@ -119,15 +120,25 @@ class PHPUnit_Framework_MockObject_InvocationMocker implements PHPUnit_Framework
             $returnValue = NULL;
         }
 
-        foreach($this->matchers as $match) {
-            if ($match->matches($invocation)) {
-                $value = $match->invoked($invocation);
+        foreach ($this->matchers as $match) {
+            try {
+                if ($match->matches($invocation)) {
+                    $value = $match->invoked($invocation);
 
-                if (!$hasReturnValue) {
-                    $returnValue    = $value;
-                    $hasReturnValue = TRUE;
+                    if (!$hasReturnValue) {
+                        $returnValue    = $value;
+                        $hasReturnValue = TRUE;
+                    }
                 }
             }
+
+            catch (Exception $e) {
+                $exception = $e;
+            }
+        }
+
+        if ($exception !== NULL) {
+            throw $exception;
         }
 
         return $returnValue;
