@@ -2,7 +2,7 @@
 /**
  * PHPUnit
  *
- * Copyright (c) 2002-2008, Sebastian Bergmann <sb@sebastian-bergmann.de>.
+ * Copyright (c) 2002-2009, Sebastian Bergmann <sb@sebastian-bergmann.de>.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,16 +37,16 @@
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    SVN: $Id: ConstraintTest.php 2396 2008-02-09 16:07:59Z sb $
+ * @version    SVN: $Id: ConstraintTest.php 4661 2009-02-23 16:34:27Z sb $
  * @link       http://www.phpunit.de/
  * @since      File available since Release 3.0.0
  */
 
 require_once 'PHPUnit/Framework/TestCase.php';
 
-require_once '_files/ClassWithNonPublicAttributes.php';
+require_once dirname(dirname(__FILE__)) . DIRECTORY_SEPARATOR . '_files' . DIRECTORY_SEPARATOR . 'ClassWithNonPublicAttributes.php';
 
 /**
  *
@@ -54,9 +54,9 @@ require_once '_files/ClassWithNonPublicAttributes.php';
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    Release: 3.2.16
+ * @version    Release: 3.3.16
  * @link       http://www.phpunit.de/
  * @since      Class available since Release 3.0.0
  */
@@ -67,7 +67,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $constraint = new PHPUnit_Framework_Constraint_ArrayHasKey(0);
 
         $this->assertFalse($constraint->evaluate(array()));
-        $this->assertEquals('has key <integer:0>', $constraint->toString());
+        $this->assertEquals('has the key <integer:0>', $constraint->toString());
 
         try {
             $constraint->fail(array(), '');
@@ -75,7 +75,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "Failed asserting that \nArray\n(\n)\n has key <integer:0>.",
+              "Failed asserting that an array has the key <integer:0>.",
               $e->getDescription()
             );
 
@@ -95,7 +95,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "custom message\nFailed asserting that \nArray\n(\n)\n has key <integer:0>.",
+              "custom message\nFailed asserting that an array has the key <integer:0>.",
               $e->getDescription()
             );
 
@@ -112,15 +112,15 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         );
 
         $this->assertTrue($constraint->evaluate(array()));
-        $this->assertEquals('does not have key <integer:0>', $constraint->toString());
+        $this->assertEquals('does not have the key <integer:0>', $constraint->toString());
 
         try {
-            $constraint->fail(array(0), '');
+            $constraint->fail(array(0), '', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "Failed asserting that \nArray\n(\n    [0] => 0\n)\n does not have key <integer:0>.",
+              "Failed asserting that an array does not have the key <integer:0>.",
               $e->getDescription()
             );
 
@@ -137,12 +137,12 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         );
 
         try {
-            $constraint->fail(array(0), 'custom message');
+            $constraint->fail(array(0), 'custom message', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "custom message\nFailed asserting that \nArray\n(\n    [0] => 0\n)\n does not have key <integer:0>.",
+              "custom message\nFailed asserting that an array does not have the key <integer:0>.",
               $e->getDescription()
             );
 
@@ -205,7 +205,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $this->assertEquals('file does not exist', $constraint->toString());
 
         try {
-            $constraint->fail('foo', '');
+            $constraint->fail('foo', '', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -227,7 +227,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         );
 
         try {
-            $constraint->fail('foo', 'custom message');
+            $constraint->fail('foo', 'custom message', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -296,7 +296,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $this->assertEquals('is not greater than <integer:1>', $constraint->toString());
 
         try {
-            $constraint->fail(1, '');
+            $constraint->fail(1, '', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -318,7 +318,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         );
 
         try {
-            $constraint->fail(1, 'custom message');
+            $constraint->fail(1, 'custom message', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -408,7 +408,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $this->assertEquals('is not equal to <integer:1>', $constraint->toString());
 
         try {
-            $constraint->fail(1, '');
+            $constraint->fail(1, '', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -430,7 +430,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         );
 
         try {
-            $constraint->fail(1, 'custom message');
+            $constraint->fail(1, 'custom message', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -454,7 +454,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
 
         $this->assertFalse($constraint->evaluate($b));
         $this->assertTrue($constraint->evaluate($a));
-        $this->assertEquals("is identical to \nstdClass Object\n(\n)\n", $constraint->toString());
+        $this->assertEquals('is identical to an object of class "stdClass"', $constraint->toString());
 
         try {
             $constraint->fail($b, '');
@@ -462,7 +462,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "Failed asserting that \nstdClass Object\n(\n)\n is identical to \nstdClass Object\n(\n)\n.",
+              "Failed asserting that \nstdClass Object\n(\n)\n is identical to an object of class \"stdClass\".",
               $e->getDescription()
             );
 
@@ -485,7 +485,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "custom message\nFailed asserting that \nstdClass Object\n(\n)\n is identical to \nstdClass Object\n(\n)\n.",
+              "custom message\nFailed asserting that \nstdClass Object\n(\n)\n is identical to an object of class \"stdClass\".",
               $e->getDescription()
             );
 
@@ -506,15 +506,15 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
 
         $this->assertTrue($constraint->evaluate($b));
         $this->assertFalse($constraint->evaluate($a));
-        $this->assertEquals("is not identical to \nstdClass Object\n(\n)\n", $constraint->toString());
+        $this->assertEquals("is not identical to an object of class \"stdClass\"", $constraint->toString());
 
         try {
-            $constraint->fail($a, '');
+            $constraint->fail($a, '', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "Failed asserting that \nstdClass Object\n(\n)\n is not identical to \nstdClass Object\n(\n)\n.",
+              "Failed asserting that \nstdClass Object\n(\n)\n is not identical to an object of class \"stdClass\".",
               $e->getDescription()
             );
 
@@ -533,12 +533,12 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         );
 
         try {
-            $constraint->fail($a, 'custom message');
+            $constraint->fail($a, 'custom message', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "custom message\nFailed asserting that \nstdClass Object\n(\n)\n is not identical to \nstdClass Object\n(\n)\n.",
+              "custom message\nFailed asserting that \nstdClass Object\n(\n)\n is not identical to an object of class \"stdClass\".",
               $e->getDescription()
             );
 
@@ -603,7 +603,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $this->assertEquals('is not instance of class "stdClass"', $constraint->toString());
 
         try {
-            $constraint->fail(new stdClass, '');
+            $constraint->fail(new stdClass, '', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -625,7 +625,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         );
 
         try {
-            $constraint->fail(new stdClass, 'custom message');
+            $constraint->fail(new stdClass, 'custom message', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -695,7 +695,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $this->assertEquals('is not of type "string"', $constraint->toString());
 
         try {
-            $constraint->fail('', '');
+            $constraint->fail('', '', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -717,7 +717,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         );
 
         try {
-            $constraint->fail('', 'custom message');
+            $constraint->fail('', 'custom message', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -786,7 +786,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $this->assertEquals('is not less than <integer:1>', $constraint->toString());
 
         try {
-            $constraint->fail(1, '');
+            $constraint->fail(1, '', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -808,7 +808,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         );
 
         try {
-            $constraint->fail(1, 'custom message');
+            $constraint->fail(1, 'custom message', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -836,7 +836,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "Failed asserting that \nstdClass Object\n(\n)\n has attribute \"foo\".",
+              'Failed asserting that object of class "stdClass" has attribute "foo".',
               $e->getDescription()
             );
 
@@ -856,7 +856,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "custom message\nFailed asserting that \nstdClass Object\n(\n)\n has attribute \"foo\".",
+              "custom message\nFailed asserting that object of class \"stdClass\" has attribute \"foo\".",
               $e->getDescription()
             );
 
@@ -879,12 +879,12 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $o->foo = 'bar';
 
         try {
-            $constraint->fail($o, '');
+            $constraint->fail($o, '', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "Failed asserting that \nstdClass Object\n(\n    [foo] => bar\n)\n does not have attribute \"foo\".",
+              'Failed asserting that object of class "stdClass" does not have attribute "foo".',
               $e->getDescription()
             );
 
@@ -904,12 +904,12 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $o->foo = 'bar';
 
         try {
-            $constraint->fail($o, 'custom message');
+            $constraint->fail($o, 'custom message', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "custom message\nFailed asserting that \nstdClass Object\n(\n    [foo] => bar\n)\n does not have attribute \"foo\".",
+              "custom message\nFailed asserting that object of class \"stdClass\" does not have attribute \"foo\".",
               $e->getDescription()
             );
 
@@ -974,7 +974,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $this->assertEquals('does not match PCRE pattern "/foo/"', $constraint->toString());
 
         try {
-            $constraint->fail('barfoobar', '');
+            $constraint->fail('barfoobar', '', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -996,7 +996,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         );
 
         try {
-            $constraint->fail('barfoobar', 'custom message');
+            $constraint->fail('barfoobar', 'custom message', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -1066,7 +1066,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $this->assertEquals('does not contain "foo"', $constraint->toString());
 
         try {
-            $constraint->fail('barfoobar', '');
+            $constraint->fail('barfoobar', '', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -1088,7 +1088,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         );
 
         try {
-            $constraint->fail('barfoobar', 'custom message');
+            $constraint->fail('barfoobar', 'custom message', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
@@ -1103,7 +1103,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $this->fail();
     }
 
-    public function testConstraintTraversableContains()
+    public function testConstraintArrayContains()
     {
         $constraint = new PHPUnit_Framework_Constraint_TraversableContains('foo');
 
@@ -1117,7 +1117,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "Failed asserting that \nArray\n(\n    [0] => bar\n)\n contains <string:foo>.",
+              'Failed asserting that an array contains <string:foo>.',
               $e->getDescription()
             );
 
@@ -1127,7 +1127,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $this->fail();
     }
 
-    public function testConstraintTraversableContains2()
+    public function testConstraintArrayContains2()
     {
         $constraint = new PHPUnit_Framework_Constraint_TraversableContains('foo');
 
@@ -1137,7 +1137,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "custom message\nFailed asserting that \nArray\n(\n    [0] => bar\n)\n contains <string:foo>.",
+              "custom message\nFailed asserting that an array contains <string:foo>.",
               $e->getDescription()
             );
 
@@ -1147,7 +1147,7 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $this->fail();
     }
 
-    public function testConstraintTraversableNotContains()
+    public function testConstraintArrayNotContains()
     {
         $constraint = new PHPUnit_Framework_Constraint_Not(
           new PHPUnit_Framework_Constraint_TraversableContains('foo')
@@ -1158,12 +1158,12 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $this->assertEquals('does not contain <string:foo>', $constraint->toString());
 
         try {
-            $constraint->fail(array('foo'), '');
+            $constraint->fail(array('foo'), '', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "Failed asserting that \nArray\n(\n    [0] => foo\n)\n does not contain <string:foo>.",
+              'Failed asserting that an array does not contain <string:foo>.',
               $e->getDescription()
             );
 
@@ -1173,19 +1173,74 @@ class Framework_ConstraintTest extends PHPUnit_Framework_TestCase
         $this->fail();
     }
 
-    public function testConstraintTraversableNotContains2()
+    public function testConstraintArrayNotContains2()
     {
         $constraint = new PHPUnit_Framework_Constraint_Not(
           new PHPUnit_Framework_Constraint_TraversableContains('foo')
         );
 
         try {
-            $constraint->fail(array('foo'), 'custom message');
+            $constraint->fail(array('foo'), 'custom message', TRUE);
         }
 
         catch (PHPUnit_Framework_ExpectationFailedException $e) {
             $this->assertEquals(
-              "custom message\nFailed asserting that \nArray\n(\n    [0] => foo\n)\n does not contain <string:foo>.",
+              "custom message\nFailed asserting that an array does not contain <string:foo>.",
+              $e->getDescription()
+            );
+
+            return;
+        }
+
+        $this->fail();
+    }
+
+    /**
+     * @covers PHPUnit_Framework_Constraint_TraversableContains
+     */
+    public function testConstraintSplObjectStorageContains()
+    {
+        $object     = new StdClass;
+        $constraint = new PHPUnit_Framework_Constraint_TraversableContains($object);
+        $this->assertEquals("contains \nstdClass Object\n(\n)\n", $constraint->toString());
+
+        $storage = new SplObjectStorage;
+        $this->assertFalse($constraint->evaluate($storage));
+
+        $storage->attach($object);
+        $this->assertTrue($constraint->evaluate($storage));
+
+        try {
+            $constraint->fail(new SplObjectStorage, '');
+        }
+
+        catch (PHPUnit_Framework_ExpectationFailedException $e) {
+            $this->assertEquals(
+              "Failed asserting that an iterator contains \nstdClass Object\n(\n)\n.",
+              $e->getDescription()
+            );
+
+            return;
+        }
+
+        $this->fail();
+    }
+
+    /**
+     * @covers PHPUnit_Framework_Constraint_TraversableContains
+     */
+    public function testConstraintSplObjectStorageContains2()
+    {
+        $object     = new StdClass;
+        $constraint = new PHPUnit_Framework_Constraint_TraversableContains($object);
+
+        try {
+            $constraint->fail(new SplObjectStorage, 'custom message');
+        }
+
+        catch (PHPUnit_Framework_ExpectationFailedException $e) {
+            $this->assertEquals(
+              "custom message\nFailed asserting that an iterator contains \nstdClass Object\n(\n)\n.",
               $e->getDescription()
             );
 
