@@ -2,7 +2,7 @@
 /**
  * PHPUnit
  *
- * Copyright (c) 2002-2008, Sebastian Bergmann <sb@sebastian-bergmann.de>.
+ * Copyright (c) 2002-2009, Sebastian Bergmann <sb@sebastian-bergmann.de>.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,9 +37,9 @@
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    SVN: $Id: TestRunner.php 2567 2008-03-01 07:41:20Z sb $
+ * @version    SVN: $Id: TestRunner.php 4404 2008-12-31 09:27:18Z sb $
  * @link       http://www.phpunit.de/
  * @since      File available since Release 2.0.0
  */
@@ -50,22 +50,12 @@ require_once 'PHPUnit/Extensions/RepeatedTest.php';
 require_once 'PHPUnit/Runner/StandardTestSuiteLoader.php';
 require_once 'PHPUnit/Runner/Version.php';
 require_once 'PHPUnit/TextUI/ResultPrinter.php';
-require_once 'PHPUnit/Util/TestDox/ResultPrinter.php';
 require_once 'PHPUnit/Util/Configuration.php';
 require_once 'PHPUnit/Util/PDO.php';
+require_once 'PHPUnit/Util/Filesystem.php';
 require_once 'PHPUnit/Util/Filter.php';
 require_once 'PHPUnit/Util/Report.php';
 require_once 'PHPUnit/Util/Timer.php';
-require_once 'PHPUnit/Util/Log/CodeCoverage/Database.php';
-require_once 'PHPUnit/Util/Log/CodeCoverage/XML.php';
-require_once 'PHPUnit/Util/Log/CPD.php';
-require_once 'PHPUnit/Util/Log/Database.php';
-require_once 'PHPUnit/Util/Log/GraphViz.php';
-require_once 'PHPUnit/Util/Log/JSON.php';
-require_once 'PHPUnit/Util/Log/Metrics.php';
-require_once 'PHPUnit/Util/Log/TAP.php';
-require_once 'PHPUnit/Util/Log/PMD.php';
-require_once 'PHPUnit/Util/Log/XML.php';
 
 PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');
 
@@ -76,9 +66,9 @@ PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    Release: 3.2.16
+ * @version    Release: 3.3.16
  * @link       http://www.phpunit.de/
  * @since      Class available since Release 2.0.0
  */
@@ -90,21 +80,16 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
 
     /**
      * @var    PHPUnit_Runner_TestSuiteLoader
-     * @access protected
-     * @static
      */
     protected static $loader = NULL;
 
     /**
      * @var    PHPUnit_TextUI_ResultPrinter
-     * @access protected
      */
     protected $printer = NULL;
 
     /**
      * @var    boolean
-     * @access protected
-     * @static
      */
     protected static $versionStringPrinted = FALSE;
 
@@ -112,8 +97,6 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
      * @param  mixed $test
      * @param  array $arguments
      * @throws InvalidArgumentException
-     * @access public
-     * @static
      */
     public static function run($test, array $arguments = array())
     {
@@ -139,8 +122,6 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
      * Runs a single test and waits until the user types RETURN.
      *
      * @param  PHPUnit_Framework_Test $suite
-     * @access public
-     * @static
      */
     public static function runAndWait(PHPUnit_Framework_Test $suite)
     {
@@ -157,7 +138,6 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
 
     /**
      * @return PHPUnit_Framework_TestResult
-     * @access protected
      */
     protected function createTestResult()
     {
@@ -168,12 +148,15 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
      * @param  PHPUnit_Framework_Test $suite
      * @param  array                  $arguments
      * @return PHPUnit_Framework_TestResult
-     * @access public
      */
     public function doRun(PHPUnit_Framework_Test $suite, array $arguments = array())
     {
         $this->handleConfiguration($arguments);
 
+        if (isset($arguments['bootstrap'])) {
+            PHPUnit_Util_Fileloader::load($arguments['bootstrap']);
+        }
+
         if (is_integer($arguments['repeat'])) {
             $suite = new PHPUnit_Extensions_RepeatedTest(
               $suite,
@@ -191,7 +174,11 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
         }
 
         if (!$arguments['convertNoticesToExceptions']) {
-            PHPUnit_Framework_Notice::$enabled = FALSE;
+            PHPUnit_Framework_Error_Notice::$enabled = FALSE;
+        }
+
+        if (!$arguments['convertWarningsToExceptions']) {
+            PHPUnit_Framework_Error_Warning::$enabled = FALSE;
         }
 
         if ($arguments['stopOnFailure']) {
@@ -204,14 +191,16 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
                 $this->printer = $arguments['printer'];
             } else {
                 $this->printer = new PHPUnit_TextUI_ResultPrinter(
-                  NULL, $arguments['verbose']
+                  NULL, $arguments['verbose'], $arguments['colors'], $arguments['debug']
                 );
             }
         }
 
-        $this->printer->write(
-          PHPUnit_Runner_Version::getVersionString() . "\n\n"
-        );
+        if (!$this->printer instanceof PHPUnit_Util_Log_TAP) {
+            $this->printer->write(
+              PHPUnit_Runner_Version::getVersionString() . "\n\n"
+            );
+        }
 
         foreach ($arguments['listeners'] as $listener) {
             $result->addListener($listener);
@@ -219,7 +208,29 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
 
         $result->addListener($this->printer);
 
+        if (isset($arguments['storyHTMLFile'])) {
+            require_once 'PHPUnit/Extensions/Story/ResultPrinter/HTML.php';
+
+            $result->addListener(
+              new PHPUnit_Extensions_Story_ResultPrinter_HTML(
+                $arguments['storyHTMLFile']
+              )
+            );
+        }
+
+        if (isset($arguments['storyTextFile'])) {
+            require_once 'PHPUnit/Extensions/Story/ResultPrinter/Text.php';
+
+            $result->addListener(
+              new PHPUnit_Extensions_Story_ResultPrinter_Text(
+                $arguments['storyTextFile']
+              )
+            );
+        }
+
         if (isset($arguments['testdoxHTMLFile'])) {
+            require_once 'PHPUnit/Util/TestDox/ResultPrinter/HTML.php';
+
             $result->addListener(
               new PHPUnit_Util_TestDox_ResultPrinter_HTML(
                 $arguments['testdoxHTMLFile']
@@ -228,6 +239,8 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
         }
 
         if (isset($arguments['testdoxTextFile'])) {
+            require_once 'PHPUnit/Util/TestDox/ResultPrinter/Text.php';
+
             $result->addListener(
               new PHPUnit_Util_TestDox_ResultPrinter_Text(
                 $arguments['testdoxTextFile']
@@ -237,37 +250,42 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
 
         if (isset($arguments['graphvizLogfile'])) {
             if (PHPUnit_Util_Filesystem::fileExistsInIncludePath('Image/GraphViz.php')) {
+                require_once 'PHPUnit/Util/Log/GraphViz.php';
+
                 $result->addListener(
                   new PHPUnit_Util_Log_GraphViz($arguments['graphvizLogfile'])
                 );
             }
         }
 
-        if ((isset($arguments['coverageXML']) ||
-             isset($arguments['metricsXML'])  ||
-             isset($arguments['pmdXML'])) &&
+        if ((isset($arguments['coverageClover']) ||
+             isset($arguments['coverageSource']) ||
+             isset($arguments['metricsXML']) ||
+             isset($arguments['pmdXML']) ||
+             isset($arguments['reportDirectory'])) &&
              extension_loaded('xdebug')) {
             $result->collectCodeCoverageInformation(TRUE);
         }
 
-        if (isset($arguments['reportDirectory']) &&
-            extension_loaded('xdebug')) {
-            $result->collectCodeCoverageInformation(TRUE);
-        }
-
         if (isset($arguments['jsonLogfile'])) {
+            require_once 'PHPUnit/Util/Log/JSON.php';
+
             $result->addListener(
               new PHPUnit_Util_Log_JSON($arguments['jsonLogfile'])
             );
         }
 
         if (isset($arguments['tapLogfile'])) {
+            require_once 'PHPUnit/Util/Log/TAP.php';
+
             $result->addListener(
               new PHPUnit_Util_Log_TAP($arguments['tapLogfile'])
             );
         }
 
         if (isset($arguments['xmlLogfile'])) {
+            require_once 'PHPUnit/Util/Log/XML.php';
+
             $result->addListener(
               new PHPUnit_Util_Log_XML(
                 $arguments['xmlLogfile'], $arguments['logIncompleteSkipped']
@@ -286,6 +304,8 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
         if ($writeToTestDatabase) {
             $dbh = PHPUnit_Util_PDO::factory($arguments['testDatabaseDSN']);
 
+            require_once 'PHPUnit/Util/Log/Database.php';
+
             $dbListener = PHPUnit_Util_Log_Database::getInstance(
               $dbh,
               $arguments['testDatabaseLogRevision'],
@@ -309,84 +329,102 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
             $this->printer->printResult($result);
         }
 
-        if (isset($arguments['coverageXML']) &&
-            extension_loaded('tokenizer') &&
-            extension_loaded('xdebug')) {
-            $this->printer->write("\nWriting code coverage data to XML file, this may take a moment.");
+        if (extension_loaded('tokenizer') && extension_loaded('xdebug')) {
+            if (isset($arguments['coverageClover'])) {
+                $this->printer->write("\nWriting code coverage data to XML file, this may take a moment.");
 
-            $writer = new PHPUnit_Util_Log_CodeCoverage_XML(
-              $arguments['coverageXML']
-            );
+                require_once 'PHPUnit/Util/Log/CodeCoverage/XML/Clover.php';
 
-            $writer->process($result);
-            $this->printer->write("\n");
-        }
+                $writer = new PHPUnit_Util_Log_CodeCoverage_XML_Clover(
+                  $arguments['coverageClover']
+                );
 
-        if ($writeToTestDatabase &&
-            extension_loaded('tokenizer') &&
-            extension_loaded('xdebug')) {
-            $this->printer->write("\nStoring code coverage and software metrics data in database.\nThis may take a moment.");
+                $writer->process($result);
+                $this->printer->write("\n");
+            }
 
-            $testDb = new PHPUnit_Util_Log_CodeCoverage_Database($dbh);
-            $testDb->storeCodeCoverage(
-              $result,
-              $dbListener->getRunId(),
-              $arguments['testDatabaseLogRevision'],
-              $arguments['testDatabasePrefix']
-            );
+            if (isset($arguments['coverageSource'])) {
+                $this->printer->write("\nWriting code coverage data to XML files, this may take a moment.");
 
-            $this->printer->write("\n");
-        }
+                require_once 'PHPUnit/Util/Log/CodeCoverage/XML/Source.php';
 
-        if (isset($arguments['metricsXML']) &&
-            extension_loaded('tokenizer') &&
-            extension_loaded('xdebug')) {
-            $this->printer->write("\nWriting metrics report XML file, this may take a moment.");
+                $writer = new PHPUnit_Util_Log_CodeCoverage_XML_Source(
+                  $arguments['coverageSource']
+                );
 
-            $writer = new PHPUnit_Util_Log_Metrics(
-              $arguments['metricsXML']
-            );
+                $writer->process($result);
+                $this->printer->write("\n");
+            }
 
-            $writer->process($result);
-            $this->printer->write("\n");
-        }
+            if ($writeToTestDatabase) {
+                $this->printer->write("\nStoring code coverage and software metrics data in database.\nThis may take a moment.");
 
-        if (isset($arguments['pmdXML']) &&
-            extension_loaded('tokenizer') &&
-            extension_loaded('xdebug')) {
-            $writer = new PHPUnit_Util_Log_PMD(
-              $arguments['pmdXML'], $arguments['pmd']
-            );
+                require_once 'PHPUnit/Util/Log/CodeCoverage/Database.php';
 
-            $this->printer->write("\nWriting violations report XML file, this may take a moment.");
-            $writer->process($result);
+                $testDb = new PHPUnit_Util_Log_CodeCoverage_Database($dbh);
+                $testDb->storeCodeCoverage(
+                  $result,
+                  $dbListener->getRunId(),
+                  $arguments['testDatabaseLogRevision'],
+                  $arguments['testDatabasePrefix']
+                );
 
-            $writer = new PHPUnit_Util_Log_CPD(
-              str_replace('.xml', '-cpd.xml', $arguments['pmdXML'])
-            );
+                $this->printer->write("\n");
+            }
 
-            $writer->process(
-              $result, $arguments['cpdMinLines'], $arguments['cpdMinMatches']
-            );
+            if (isset($arguments['metricsXML'])) {
+                $this->printer->write("\nWriting metrics report XML file, this may take a moment.");
 
-            $this->printer->write("\n");
-        }
+                require_once 'PHPUnit/Util/Log/Metrics.php';
 
-        if (isset($arguments['reportDirectory']) &&
-            extension_loaded('xdebug')) {
-            $this->printer->write("\nGenerating code coverage report, this may take a moment.");
+                $writer = new PHPUnit_Util_Log_Metrics(
+                  $arguments['metricsXML']
+                );
 
-            PHPUnit_Util_Report::render(
-              $result,
-              $arguments['reportDirectory'],
-              $arguments['reportCharset'],
-              $arguments['reportYUI'],
-              $arguments['reportHighlight'],
-              $arguments['reportLowUpperBound'],
-              $arguments['reportHighLowerBound']
-            );
+                $writer->process($result);
+                $this->printer->write("\n");
+            }
 
-            $this->printer->write("\n");
+            if (isset($arguments['pmdXML'])) {
+                require_once 'PHPUnit/Util/Log/PMD.php';
+
+                $writer = new PHPUnit_Util_Log_PMD(
+                  $arguments['pmdXML'], $arguments['pmd']
+                );
+
+                $this->printer->write("\nWriting violations report XML file, this may take a moment.");
+                $writer->process($result);
+
+                require_once 'PHPUnit/Util/Log/CPD.php';
+
+                $writer = new PHPUnit_Util_Log_CPD(
+                  str_replace('.xml', '-cpd.xml', $arguments['pmdXML'])
+                );
+
+                $writer->process(
+                  $result, $arguments['cpdMinLines'], $arguments['cpdMinMatches']
+                );
+
+                $this->printer->write("\n");
+            }
+
+            if (isset($arguments['reportDirectory'])) {
+                $this->printer->write("\nGenerating code coverage report, this may take a moment.");
+
+                unset($suite);
+
+                PHPUnit_Util_Report::render(
+                  $result,
+                  $arguments['reportDirectory'],
+                  $arguments['reportCharset'],
+                  $arguments['reportYUI'],
+                  $arguments['reportHighlight'],
+                  $arguments['reportLowUpperBound'],
+                  $arguments['reportHighLowerBound']
+                );
+
+                $this->printer->write("\n");
+            }
         }
 
         $this->pause($arguments['wait']);
@@ -396,7 +434,6 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
 
     /**
      * @param  boolean $wait
-     * @access protected
      */
     protected function pause($wait)
     {
@@ -413,7 +450,6 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
 
     /**
      * @param  PHPUnit_TextUI_ResultPrinter $resultPrinter
-     * @access public
      */
     public function setPrinter(PHPUnit_TextUI_ResultPrinter $resultPrinter)
     {
@@ -424,7 +460,6 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
      * A test started.
      *
      * @param  string  $testName
-     * @access public
      */
     public function testStarted($testName)
     {
@@ -434,7 +469,6 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
      * A test ended.
      *
      * @param  string  $testName
-     * @access public
      */
     public function testEnded($testName)
     {
@@ -446,7 +480,6 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
      * @param  integer                                 $status
      * @param  PHPUnit_Framework_Test                 $test
      * @param  PHPUnit_Framework_AssertionFailedError $e
-     * @access public
      */
     public function testFailed($status, PHPUnit_Framework_Test $test, PHPUnit_Framework_AssertionFailedError $e)
     {
@@ -457,7 +490,6 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
      * a test suite.
      *
      * @param  string  $message
-     * @access protected
      */
     protected function runFailed($message)
     {
@@ -466,40 +498,14 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
         exit(self::FAILURE_EXIT);
     }
 
-    /**
-     * @param  string $directory
-     * @return string
-     * @throws RuntimeException
-     * @access protected
-     * @since  Method available since Release 3.0.0
-     */
-    protected function getDirectory($directory)
-    {
-        if (substr($directory, -1, 1) != DIRECTORY_SEPARATOR) {
-            $directory .= DIRECTORY_SEPARATOR;
-        }
-
-        if (is_dir($directory) || mkdir($directory, 0777, TRUE)) {
-            return $directory;
-        } else {
-            throw new RuntimeException(
-              sprintf(
-                'Directory "%s" does not exist.',
-                $directory
-              )
-            );
-        }
-    }
-
     /**
      * @param  string $buffer
-     * @access protected
      * @since  Method available since Release 3.1.0
      */
     protected static function write($buffer)
     {
-        if (php_sapi_name() != 'cli') {
-            $buffer = htmlentities($buffer);
+        if (PHP_SAPI != 'cli') {
+            $buffer = htmlspecialchars($buffer);
         }
 
         print $buffer;
@@ -509,7 +515,6 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
      * Returns the loader to be used.
      *
      * @return PHPUnit_Runner_TestSuiteLoader
-     * @access public
      * @since  Method available since Release 2.2.0
      */
     public function getLoader()
@@ -525,8 +530,6 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
      * Sets the loader to be used.
      *
      * @param PHPUnit_Runner_TestSuiteLoader $loader
-     * @access public
-     * @static
      * @since  Method available since Release 3.0.0
      */
     public static function setLoader(PHPUnit_Runner_TestSuiteLoader $loader)
@@ -535,7 +538,6 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
     }
 
     /**
-     * @access public
      */
     public static function showError($message)
     {
@@ -547,8 +549,6 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
 
 
     /**
-     * @access public
-     * @static
      */
     public static function printVersionString()
     {
@@ -560,7 +560,6 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
 
     /**
      * @param  array $arguments
-     * @access protected
      * @since  Method available since Release 3.2.1
      */
     protected function handleConfiguration(array &$arguments)
@@ -575,6 +574,7 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
             $arguments['pmd'] = array();
         }
 
+        $arguments['debug']              = isset($arguments['debug'])              ? $arguments['debug']              : FALSE;
         $arguments['filter']             = isset($arguments['filter'])             ? $arguments['filter']             : FALSE;
         $arguments['listeners']          = isset($arguments['listeners'])          ? $arguments['listeners']          : array();
         $arguments['repeat']             = isset($arguments['repeat'])             ? $arguments['repeat']             : FALSE;
@@ -583,6 +583,8 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
         $arguments['wait']               = isset($arguments['wait'])               ? $arguments['wait']               : FALSE;
 
         if (isset($arguments['configuration'])) {
+            $arguments['configuration']->handlePHPConfiguration();
+
             $filterConfiguration = $arguments['configuration']->getFilterConfiguration();
 
             PHPUnit_Util_Filter::$addUncoveredFilesFromWhitelist = $filterConfiguration['whitelist']['addUncoveredFilesFromWhitelist'];
@@ -629,6 +631,14 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
 
             $phpunitConfiguration = $arguments['configuration']->getPHPUnitConfiguration();
 
+            if (isset($phpunitConfiguration['bootstrap']) && !isset($arguments['bootstrap'])) {
+                $arguments['bootstrap'] = $phpunitConfiguration['bootstrap'];
+            }
+
+            if (isset($phpunitConfiguration['colors']) && !isset($arguments['colors'])) {
+                $arguments['colors'] = $phpunitConfiguration['colors'];
+            }
+
             if (isset($phpunitConfiguration['convertErrorsToExceptions']) && !isset($arguments['convertErrorsToExceptions'])) {
                 $arguments['convertErrorsToExceptions'] = $phpunitConfiguration['convertErrorsToExceptions'];
             }
@@ -637,20 +647,14 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
                 $arguments['convertNoticesToExceptions'] = $phpunitConfiguration['convertNoticesToExceptions'];
             }
 
+            if (isset($phpunitConfiguration['convertWarningsToExceptions']) && !isset($arguments['convertWarningsToExceptions'])) {
+                $arguments['convertWarningsToExceptions'] = $phpunitConfiguration['convertWarningsToExceptions'];
+            }
+
             if (isset($phpunitConfiguration['stopOnFailure']) && !isset($arguments['stopOnFailure'])) {
                 $arguments['stopOnFailure'] = $phpunitConfiguration['stopOnFailure'];
             }
 
-            $phpConfiguration = $arguments['configuration']->getPHPConfiguration();
-
-            foreach ($phpConfiguration['ini'] as $name => $value) {
-                ini_set($name, $value);
-            }
-
-            foreach ($phpConfiguration['var'] as $name => $value) {
-                $GLOBALS[$name] = $value;
-            }
-
             $groupConfiguration = $arguments['configuration']->getGroupConfiguration();
 
             if (!empty($groupConfiguration['include']) && !isset($arguments['groups'])) {
@@ -687,8 +691,16 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
                 $arguments['reportDirectory'] = $loggingConfiguration['coverage-html'];
             }
 
-            if (isset($loggingConfiguration['coverage-xml']) && !isset($arguments['coverageXML'])) {
-                $arguments['coverageXML'] = $loggingConfiguration['coverage-xml'];
+            if (isset($loggingConfiguration['coverage-clover']) && !isset($arguments['coverageClover'])) {
+                $arguments['coverageClover'] = $loggingConfiguration['coverage-clover'];
+            }
+
+            if (isset($loggingConfiguration['coverage-xml']) && !isset($arguments['coverageClover'])) {
+                $arguments['coverageClover'] = $loggingConfiguration['coverage-xml'];
+            }
+
+            if (isset($loggingConfiguration['coverage-source']) && !isset($arguments['coverageSource'])) {
+                $arguments['coverageSource'] = $loggingConfiguration['coverage-source'];
             }
 
             if (isset($loggingConfiguration['graphviz']) && !isset($arguments['graphvizLogfile'])) {
@@ -731,6 +743,14 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
                 }
             }
 
+            if (isset($loggingConfiguration['story-html']) && !isset($arguments['storyHTMLFile'])) {
+                $arguments['storyHTMLFile'] = $loggingConfiguration['story-html'];
+            }
+
+            if (isset($loggingConfiguration['story-text']) && !isset($arguments['storyTextFile'])) {
+                $arguments['storsTextFile'] = $loggingConfiguration['story-text'];
+            }
+
             if (isset($loggingConfiguration['testdox-html']) && !isset($arguments['testdoxHTMLFile'])) {
                 $arguments['testdoxHTMLFile'] = $loggingConfiguration['testdox-html'];
             }
@@ -738,31 +758,26 @@ class PHPUnit_TextUI_TestRunner extends PHPUnit_Runner_BaseTestRunner
             if (isset($loggingConfiguration['testdox-text']) && !isset($arguments['testdoxTextFile'])) {
                 $arguments['testdoxTextFile'] = $loggingConfiguration['testdox-text'];
             }
-
-            $browsers = $arguments['configuration']->getSeleniumBrowserConfiguration();
-
-            if (!empty($browsers)) {
-                require_once 'PHPUnit/Extensions/SeleniumTestCase.php';
-                PHPUnit_Extensions_SeleniumTestCase::$browsers = $browsers;
-            }
         }
 
-        $arguments['cpdMinLines']                = isset($arguments['cpdMinLines'])                ? $arguments['cpdMinLines']                : 5;
-        $arguments['cpdMinMatches']              = isset($arguments['cpdMinMatches'])              ? $arguments['cpdMinMatches']              : 70;
-        $arguments['convertErrorsToExceptions']  = isset($arguments['convertErrorsToExceptions'])  ? $arguments['convertErrorsToExceptions']  : TRUE;
-        $arguments['convertNoticesToExceptions'] = isset($arguments['convertNoticesToExceptions']) ? $arguments['convertNoticesToExceptions'] : TRUE;
-        $arguments['excludeGroups']              = isset($arguments['excludeGroups'])              ? $arguments['excludeGroups']              : array();
-        $arguments['groups']                     = isset($arguments['groups'])                     ? $arguments['groups']                     : array();
-        $arguments['logIncompleteSkipped']       = isset($arguments['logIncompleteSkipped'])       ? $arguments['logIncompleteSkipped']       : FALSE;
-        $arguments['reportCharset']              = isset($arguments['reportCharset'])              ? $arguments['reportCharset']              : 'ISO-8859-1';
-        $arguments['reportHighlight']            = isset($arguments['reportHighlight'])            ? $arguments['reportHighlight']            : FALSE;
-        $arguments['reportHighLowerBound']       = isset($arguments['reportHighLowerBound'])       ? $arguments['reportHighLowerBound']       : 70;
-        $arguments['reportLowUpperBound']        = isset($arguments['reportLowUpperBound'])        ? $arguments['reportLowUpperBound']        : 35;
-        $arguments['reportYUI']                  = isset($arguments['reportYUI'])                  ? $arguments['reportYUI']                  : TRUE;
-        $arguments['stopOnFailure']              = isset($arguments['stopOnFailure'])              ? $arguments['stopOnFailure']              : FALSE;
+        $arguments['cpdMinLines']                 = isset($arguments['cpdMinLines'])                 ? $arguments['cpdMinLines']                 : 5;
+        $arguments['cpdMinMatches']               = isset($arguments['cpdMinMatches'])               ? $arguments['cpdMinMatches']               : 70;
+        $arguments['colors']                      = isset($arguments['colors'])                      ? $arguments['colors']                      : FALSE;
+        $arguments['convertErrorsToExceptions']   = isset($arguments['convertErrorsToExceptions'])   ? $arguments['convertErrorsToExceptions']   : TRUE;
+        $arguments['convertNoticesToExceptions']  = isset($arguments['convertNoticesToExceptions'])  ? $arguments['convertNoticesToExceptions']  : TRUE;
+        $arguments['convertWarningsToExceptions'] = isset($arguments['convertWarningsToExceptions']) ? $arguments['convertWarningsToExceptions'] : TRUE;
+        $arguments['excludeGroups']               = isset($arguments['excludeGroups'])               ? $arguments['excludeGroups']               : array();
+        $arguments['groups']                      = isset($arguments['groups'])                      ? $arguments['groups']                      : array();
+        $arguments['logIncompleteSkipped']        = isset($arguments['logIncompleteSkipped'])        ? $arguments['logIncompleteSkipped']        : FALSE;
+        $arguments['reportCharset']               = isset($arguments['reportCharset'])               ? $arguments['reportCharset']               : 'ISO-8859-1';
+        $arguments['reportHighlight']             = isset($arguments['reportHighlight'])             ? $arguments['reportHighlight']             : FALSE;
+        $arguments['reportHighLowerBound']        = isset($arguments['reportHighLowerBound'])        ? $arguments['reportHighLowerBound']        : 70;
+        $arguments['reportLowUpperBound']         = isset($arguments['reportLowUpperBound'])         ? $arguments['reportLowUpperBound']         : 35;
+        $arguments['reportYUI']                   = isset($arguments['reportYUI'])                   ? $arguments['reportYUI']                   : TRUE;
+        $arguments['stopOnFailure']               = isset($arguments['stopOnFailure'])               ? $arguments['stopOnFailure']               : FALSE;
 
-        if (isset($arguments['reportDirectory'])) {
-            $arguments['reportDirectory'] = $this->getDirectory($arguments['reportDirectory']);
+        if ($arguments['filter'] !== FALSE && preg_match('/^[a-zA-Z0-9_]/', $arguments['filter'])) {
+            $arguments['filter'] = '/' . $arguments['filter'] . '/';
         }
     }
 }
