@@ -2,7 +2,7 @@
 /**
  * PHPUnit
  *
- * Copyright (c) 2002-2008, Sebastian Bergmann <sb@sebastian-bergmann.de>.
+ * Copyright (c) 2002-2009, Sebastian Bergmann <sb@sebastian-bergmann.de>.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -11,7 +11,7 @@
  *
  *   * Redistributions of source code must retain the above copyright
  *     notice, this list of conditions and the following disclaimer.
- * 
+ *
  *   * Redistributions in binary form must reproduce the above copyright
  *     notice, this list of conditions and the following disclaimer in
  *     the documentation and/or other materials provided with the
@@ -37,9 +37,9 @@
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    SVN: $Id: Configuration.php 2611 2008-03-17 16:36:51Z sb $
+ * @version    SVN: $Id: Configuration.php 4649 2009-02-18 09:29:31Z sb $
  * @link       http://www.phpunit.de/
  * @since      File available since Release 3.2.0
  */
@@ -57,7 +57,12 @@ PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');
  * <code>
  * <?xml version="1.0" encoding="utf-8" ?>
  *
- * <phpunit convertErrorsToExceptions="true" convertNoticesToExceptions="true" stopOnFailure="false">
+ * <phpunit bootstrap="/path/to/bootstrap.php"
+ *          colors="false"
+ *          convertErrorsToExceptions="true"
+ *          convertNoticesToExceptions="true"
+ *          convertWarningsToExceptions="true"
+ *          stopOnFailure="false">
  *   <testsuite name="My Test Suite">
  *     <directory suffix="Test.php">/path/to/files</directory>
  *     <file>/path/to/MyTest.php</file>
@@ -95,7 +100,8 @@ PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');
  *     <log type="coverage-html" target="/tmp/report" charset="UTF-8"
  *          yui="true" highlight="false"
  *          lowUpperBound="35" highLowerBound="70"/>
- *     <log type="coverage-xml" target="/tmp/coverage.xml"/>
+ *     <log type="coverage-clover" target="/tmp/clover.xml"/>
+ *     <log type="coverage-source" target="/tmp/coverage"/>
  *     <log type="graphviz" target="/tmp/logfile.dot"/>
  *     <log type="json" target="/tmp/logfile.json"/>
  *     <log type="metrics-xml" target="/tmp/metrics.xml"/>
@@ -103,9 +109,11 @@ PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');
  *     <log type="pmd-xml" target="/tmp/pmd.xml" cpdMinLines="5" cpdMinMatches="70"/>
  *     <log type="tap" target="/tmp/logfile.tap"/>
  *     <log type="test-xml" target="/tmp/logfile.xml" logIncompleteSkipped="false"/>
+ *     <log type="story-html" target="/tmp/story.html"/>
+ *     <log type="story-text" target="/tmp/story.txt"/>
  *     <log type="testdox-html" target="/tmp/testdox.html"/>
  *     <log type="testdox-text" target="/tmp/testdox.txt"/>
- * 
+ *
  *     <pmd>
  *       <rule class="PHPUnit_Util_Log_PMD_Rule_Project_CRAP"
  *             threshold="5,30" priority="1"/>
@@ -140,7 +148,11 @@ PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');
  *   </php>
  *
  *   <selenium>
- *     <browser name="" browser="" host="" port="" timeout="">
+ *     <browser name="Firefox on Linux"
+ *              browser="*firefox /usr/lib/firefox/firefox-bin"
+ *              host="my.linux.box"
+ *              port="4444"
+ *              timeout="30000"/>
  *   </selenium>
  * </phpunit>
  * </code>
@@ -148,9 +160,9 @@ PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    Release: 3.2.16
+ * @version    Release: 3.3.16
  * @link       http://www.phpunit.de/
  * @since      Class available since Release 3.2.0
  */
@@ -163,11 +175,10 @@ class PHPUnit_Util_Configuration
      * Loads a PHPUnit configuration file.
      *
      * @param  string $filename
-     * @access public
      */
     public function __construct($filename)
     {
-        $this->document = PHPUnit_Util_XML::load($filename);
+        $this->document = PHPUnit_Util_XML::loadFile($filename);
         $this->xpath    = new DOMXPath($this->document);
     }
 
@@ -175,7 +186,6 @@ class PHPUnit_Util_Configuration
      * Returns the configuration for SUT filtering.
      *
      * @return array
-     * @access public
      * @since  Method available since Release 3.2.1
      */
     public function getFilterConfiguration()
@@ -237,7 +247,6 @@ class PHPUnit_Util_Configuration
      * Returns the configuration for groups.
      *
      * @return array
-     * @access public
      * @since  Method available since Release 3.2.1
      */
     public function getGroupConfiguration()
@@ -262,7 +271,6 @@ class PHPUnit_Util_Configuration
      * Returns the logging configuration.
      *
      * @return array
-     * @access public
      */
     public function getLoggingConfiguration()
     {
@@ -329,7 +337,6 @@ class PHPUnit_Util_Configuration
      * Returns the PHP configuration.
      *
      * @return array
-     * @access public
      * @since  Method available since Release 3.2.1
      */
     public function getPHPConfiguration()
@@ -364,17 +371,56 @@ class PHPUnit_Util_Configuration
         return $result;
     }
 
+    /**
+     * Handles the PHP configuration.
+     *
+     * @since  Method available since Release 3.2.20
+     */
+    public function handlePHPConfiguration()
+    {
+        $configuration = $this->getPHPConfiguration();
+
+        foreach ($configuration['ini'] as $name => $value) {
+            if (defined($value)) {
+                $value = constant($value);
+            }
+
+            ini_set($name, $value);
+        }
+
+        foreach ($configuration['var'] as $name => $value) {
+            $GLOBALS[$name] = $value;
+        }
+    }
+
     /**
      * Returns the PHPUnit configuration.
      *
      * @return array
-     * @access public
      * @since  Method available since Release 3.2.14
      */
     public function getPHPUnitConfiguration()
     {
         $result = array();
 
+        if ($this->document->documentElement->hasAttribute('colors')) {
+            $result['colors'] = $this->getBoolean(
+              (string)$this->document->documentElement->getAttribute('colors'),
+              FALSE
+            );
+        }
+
+        else if ($this->document->documentElement->hasAttribute('ansi')) {
+            $result['colors'] = $this->getBoolean(
+              (string)$this->document->documentElement->getAttribute('ansi'),
+              FALSE
+            );
+        }
+
+        if ($this->document->documentElement->hasAttribute('bootstrap')) {
+            $result['bootstrap'] = (string)$this->document->documentElement->getAttribute('bootstrap');
+        }
+
         if ($this->document->documentElement->hasAttribute('convertErrorsToExceptions')) {
             $result['convertErrorsToExceptions'] = $this->getBoolean(
               (string)$this->document->documentElement->getAttribute('convertErrorsToExceptions'),
@@ -389,6 +435,13 @@ class PHPUnit_Util_Configuration
             );
         }
 
+        if ($this->document->documentElement->hasAttribute('convertWarningsToExceptions')) {
+            $result['convertWarningsToExceptions'] = $this->getBoolean(
+              (string)$this->document->documentElement->getAttribute('convertWarningsToExceptions'),
+              TRUE
+            );
+        }
+
         if ($this->document->documentElement->hasAttribute('stopOnFailure')) {
             $result['stopOnFailure'] = $this->getBoolean(
               (string)$this->document->documentElement->getAttribute('stopOnFailure'),
@@ -403,7 +456,6 @@ class PHPUnit_Util_Configuration
      * Returns the configuration for PMD rules.
      *
      * @return array
-     * @access public
      */
     public function getPMDConfiguration()
     {
@@ -434,7 +486,6 @@ class PHPUnit_Util_Configuration
      * Returns the SeleniumTestCase browser configuration.
      *
      * @return array
-     * @access public
      * @since  Method available since Release 3.2.9
      */
     public function getSeleniumBrowserConfiguration()
@@ -452,15 +503,15 @@ class PHPUnit_Util_Configuration
             }
 
             if ($config->hasAttribute('port')) {
-                $host = (int)$config->getAttribute('port');
+                $port = (int)$config->getAttribute('port');
             } else {
-                $host = 4444;
+                $port = 4444;
             }
 
             if ($config->hasAttribute('timeout')) {
-                $host = (int)$config->getAttribute('timeout');
+                $timeout = (int)$config->getAttribute('timeout');
             } else {
-                $host = 30000;
+                $timeout = 30000;
             }
 
             $result[] = array(
@@ -478,11 +529,11 @@ class PHPUnit_Util_Configuration
     /**
      * Returns the test suite configuration.
      *
+     * @param  boolean $syntaxCheck
      * @return PHPUnit_Framework_TestSuite
-     * @access public
      * @since  Method available since Release 3.2.1
      */
-    public function getTestSuiteConfiguration()
+    public function getTestSuiteConfiguration($syntaxCheck = TRUE)
     {
         $testSuiteNode = $this->xpath->query('testsuite');
 
@@ -509,11 +560,13 @@ class PHPUnit_Util_Configuration
                   $suffix
                 );
 
-                $suite->addTestFiles($testCollector->collectTests());
+                $suite->addTestFiles(
+                  $testCollector->collectTests(), $syntaxCheck
+                );
             }
 
             foreach ($this->xpath->query('testsuite/file') as $fileNode) {
-                $suite->addTestFile((string)$fileNode->nodeValue);
+                $suite->addTestFile((string)$fileNode->nodeValue, $syntaxCheck);
             }
 
             return $suite;
@@ -524,7 +577,6 @@ class PHPUnit_Util_Configuration
      * @param  string  $value
      * @param  boolean $default
      * @return boolean
-     * @access protected
      * @since  Method available since Release 3.2.3
      */
     protected function getBoolean($value, $default)
@@ -543,7 +595,6 @@ class PHPUnit_Util_Configuration
     /**
      * @param  string $query
      * @return array
-     * @access protected
      * @since  Method available since Release 3.2.3
      */
     protected function readFilterDirectories($query)
@@ -569,7 +620,6 @@ class PHPUnit_Util_Configuration
     /**
      * @param  string $query
      * @return array
-     * @access protected
      * @since  Method available since Release 3.2.3
      */
     protected function readFilterFiles($query)
