@@ -2,7 +2,7 @@
 /**
  * PHPUnit
  *
- * Copyright (c) 2002-2008, Sebastian Bergmann <sb@sebastian-bergmann.de>.
+ * Copyright (c) 2002-2009, Sebastian Bergmann <sb@sebastian-bergmann.de>.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,9 +37,9 @@
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    SVN: $Id: ErrorHandler.php 2439 2008-02-14 08:18:31Z sb $
+ * @version    SVN: $Id: ErrorHandler.php 4404 2008-12-31 09:27:18Z sb $
  * @link       http://www.phpunit.de/
  * @since      File available since Release 2.3.0
  */
@@ -50,44 +50,81 @@ require_once 'PHPUnit/Util/Filter.php';
 PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');
 
 /**
- * @param  integer $errno
- * @param  string  $errstr
- * @param  string  $errfile
- * @param  integer $errline
- * @throws PHPUnit_Framework_Error
- * @since  Function available since Release 2.3.0
+ * Error handler that converts PHP errors and warnings to exceptions.
+ *
+ * @category   Testing
+ * @package    PHPUnit
+ * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
+ * @version    Release: 3.3.16
+ * @link       http://www.phpunit.de/
+ * @since      Class available since Release 3.3.0
  */
-function PHPUnit_Util_ErrorHandler($errno, $errstr, $errfile, $errline)
+class PHPUnit_Util_ErrorHandler
 {
-    if (!($errno & error_reporting())) {
-        return FALSE;
+    protected static $errorStack = array();
+
+    /**
+     * Returns the error stack.
+     *
+     * @return array
+     */
+    public static function getErrorStack()
+    {
+        return self::$errorStack;
     }
 
-    $trace = debug_backtrace();
-    array_shift($trace);
+    /**
+     * @param  integer $errno
+     * @param  string  $errstr
+     * @param  string  $errfile
+     * @param  integer $errline
+     * @throws PHPUnit_Framework_Error
+     */
+    public static function handleError($errno, $errstr, $errfile, $errline)
+    {
+        self::$errorStack[] = array($errno, $errstr, $errfile, $errline);
 
-    foreach ($trace as $frame) {
-        if ($frame['function'] == '__toString') {
-            return FALSE;
-        }
-    }
-
-    if ($errno == E_NOTICE || $errno == E_STRICT) {
-        if (PHPUnit_Framework_Notice::$enabled !== TRUE) {
+        if (!($errno & error_reporting())) {
             return FALSE;
         }
 
-        $exception = 'PHPUnit_Framework_Notice';
-    } else {
-        $exception = 'PHPUnit_Framework_Error';
-    }
+        if (version_compare(PHP_VERSION, '5.2.5', '>=')) {
+            $trace = debug_backtrace(FALSE);
+        } else {
+            $trace = debug_backtrace();
+        }
 
-    throw new $exception(
-      $errstr,
-      $errno,
-      $errfile,
-      $errline,
-      $trace
-    );
+        array_shift($trace);
+
+        foreach ($trace as $frame) {
+            if ($frame['function'] == '__toString') {
+                return FALSE;
+            }
+        }
+
+        if ($errno == E_NOTICE || $errno == E_STRICT) {
+            if (PHPUnit_Framework_Error_Notice::$enabled !== TRUE) {
+                return FALSE;
+            }
+
+            $exception = 'PHPUnit_Framework_Error_Notice';
+        }
+
+        else if ($errno == E_WARNING) {
+            if (PHPUnit_Framework_Error_Warning::$enabled !== TRUE) {
+                return FALSE;
+            }
+
+            $exception = 'PHPUnit_Framework_Error_Warning';
+        }
+
+        else {
+            $exception = 'PHPUnit_Framework_Error';
+        }
+
+        throw new $exception($errstr, $errno, $errfile, $errline, $trace);
+    }
 }
 ?>
