@@ -2,7 +2,7 @@
 /**
  * PHPUnit
  *
- * Copyright (c) 2002-2008, Sebastian Bergmann <sb@sebastian-bergmann.de>.
+ * Copyright (c) 2002-2009, Sebastian Bergmann <sb@sebastian-bergmann.de>.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,14 +37,15 @@
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    SVN: $Id: Fileloader.php 2092 2008-01-14 16:40:53Z sb $
+ * @version    SVN: $Id: Fileloader.php 4404 2008-12-31 09:27:18Z sb $
  * @link       http://www.phpunit.de/
  * @since      File available since Release 2.3.0
  */
 
 require_once 'PHPUnit/Util/Filter.php';
+require_once 'PHPUnit/Util/Filesystem.php';
 
 PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');
 
@@ -54,7 +55,7 @@ PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
  * @version    Release: @package_version@
  * @link       http://www.phpunit.de/
@@ -66,8 +67,6 @@ class PHPUnit_Util_Fileloader
      * Path to the PHP interpreter that is to be used.
      *
      * @var    string $phpBinary
-     * @access public
-     * @static
      */
     public static $phpBinary = NULL;
 
@@ -79,8 +78,6 @@ class PHPUnit_Util_Fileloader
      * @param  string  $filename
      * @param  boolean $syntaxCheck
      * @throws RuntimeException
-     * @access public
-     * @static
      */
     public static function checkAndLoad($filename, $syntaxCheck = TRUE)
     {
@@ -108,37 +105,30 @@ class PHPUnit_Util_Fileloader
     /**
      * Loads a PHP sourcefile.
      *
-     * When the Xdebug extension is loaded and its "xdebug.collect_vars"
-     * configuration directive is enabled, all global variables declared
-     * in the loaded PHP sourcefile will be added to $GLOBALS.
-     *
      * @param  string $filename
-     * @access protected
-     * @static
+     * @return mixed
      * @since  Method available since Release 3.0.0
      */
-    protected static function load($filename)
+    public static function load($filename)
     {
-        $xdebugLoaded      = extension_loaded('xdebug');
-        $xdebugCollectVars = $xdebugLoaded && ini_get('xdebug.collect_vars') == '1';
+        $filename = PHPUnit_Util_Filesystem::fileExistsInIncludePath($filename);
 
-        if ($xdebugCollectVars) {
-            $variables = xdebug_get_declared_vars();
-        }
+        if ($filename) {
+            $oldVariableNames = array_keys(get_defined_vars());
 
-        include_once $filename;
+            include_once $filename;
 
-        if ($xdebugCollectVars) {
-            $variables = array_values(
-              array_diff(xdebug_get_declared_vars(), $variables)
-            );
+            $newVariables     = get_defined_vars();
+            $newVariableNames = array_diff(array_keys($newVariables), $oldVariableNames);
 
-            foreach ($variables as $variable) {
-                if (isset($$variable)) {
-                    $GLOBALS[$variable] = $$variable;
+            foreach ($newVariableNames as $variableName) {
+                if ($variableName != 'oldVariableNames') {
+                    $GLOBALS[$variableName] = $newVariables[$variableName];
                 }
             }
         }
+
+        return $filename;
     }
 
     /**
@@ -168,8 +158,6 @@ class PHPUnit_Util_Fileloader
      *
      * @param  string $filename
      * @throws RuntimeException
-     * @access protected
-     * @static
      * @since  Method available since Release 3.0.0
      */
     protected static function syntaxCheck($filename)
@@ -179,14 +167,11 @@ class PHPUnit_Util_Fileloader
                 self::$phpBinary = '@php_bin@';
             }
 
-            else if (PHP_SAPI == 'cli' && isset($_SERVER['_'])) {
-                self::$phpBinary = $_SERVER['_'];
-
-                if (strpos(self::$phpBinary, 'phpunit') !== FALSE) {
-                    $file            = file(self::$phpBinary);
-                    $tmp             = explode(' ', $file[0]);
-                    self::$phpBinary = trim($tmp[1]);
-                }
+            else if (PHP_SAPI == 'cli' && isset($_SERVER['_']) &&
+                     strpos($_SERVER['_'], 'phpunit') !== FALSE) {
+                $file            = file($_SERVER['_']);
+                $tmp             = explode(' ', $file[0]);
+                self::$phpBinary = trim($tmp[1]);
             }
 
             if (!is_readable(self::$phpBinary)) {
