@@ -2,7 +2,7 @@
 /**
  * PHPUnit
  *
- * Copyright (c) 2002-2008, Sebastian Bergmann <sb@sebastian-bergmann.de>.
+ * Copyright (c) 2002-2009, Sebastian Bergmann <sb@sebastian-bergmann.de>.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,9 +37,9 @@
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    SVN: $Id: Filter.php 2041 2008-01-08 10:00:39Z sb $
+ * @version    SVN: $Id: Filter.php 4404 2008-12-31 09:27:18Z sb $
  * @link       http://www.phpunit.de/
  * @since      File available since Release 2.0.0
  */
@@ -52,9 +52,9 @@ require_once 'PHPUnit/Util/FilterIterator.php';
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    Release: 3.2.16
+ * @version    Release: 3.3.16
  * @link       http://www.phpunit.de/
  * @since      Class available since Release 2.0.0
  */
@@ -62,92 +62,51 @@ class PHPUnit_Util_Filter
 {
     /**
      * @var    boolean
-     * @access public
-     * @static
      */
     public static $addUncoveredFilesFromWhitelist = TRUE;
 
     /**
      * @var    boolean
-     * @access public
-     * @static
      */
     public static $filterPHPUnit = TRUE;
 
     /**
      * @var    boolean
-     * @access protected
-     * @static
      */
     protected static $filter = TRUE;
 
-    /**
-     * @var    boolean
-     * @access protected
-     * @static
-     */
-    protected static $blackListConverstionForWindowsDone = FALSE;
-
     /**
      * Source files that are blacklisted.
      *
      * @var    array
-     * @access protected
-     * @static
      */
     protected static $blacklistedFiles = array(
       'DEFAULT' => array(),
       'PHPUNIT' => array(),
-      'TESTS' => array(),
-      'PEAR' => array(
-        'Console/Getopt.php',
-        'Image/GraphViz.php',
-        'Log/composite.php',
-        'Log/console.php',
-        'Log/daemon.php',
-        'Log/display.php',
-        'Log/error_log.php',
-        'Log/file.php',
-        'Log/mail.php',
-        'Log/mcal.php',
-        'Log/mdb2.php',
-        'Log/null.php',
-        'Log/observer.php',
-        'Log/sql.php',
-        'Log/sqlite.php',
-        'Log/syslog.php',
-        'Log/win.php',
-        'Log.php',
-        'PEAR/Installer/Role/Common.php',
-        'PEAR/Installer/Role.php',
-        'PEAR/Config.php',
-        'PEAR/DependencyDB.php',
-        'PEAR/Registry.php',
-        'PEAR/Remote.php',
-        'PEAR/RunTest.php',
-        'PEAR/XMLParser.php',
-        'PEAR.php',
-        'System.php'
-      )
+      'TESTS' => array()
     );
 
     /**
      * Source files that are whitelisted.
      *
      * @var    array
-     * @access protected
-     * @static
      */
     protected static $whitelistedFiles = array();
 
+    /**
+     * List of covered files.
+     *
+     * @var    array
+     */
+    protected static $coveredFiles = array();
+
     /**
      * Adds a directory to the blacklist (recursively).
      *
      * @param  string $directory
      * @param  string $suffix
      * @param  string $group
-     * @access public
-     * @static
+     * @throws RuntimeException
      * @since  Method available since Release 3.1.5
      */
     public static function addDirectoryToFilter($directory, $suffix = '.php', $group = 'DEFAULT')
@@ -156,6 +115,8 @@ class PHPUnit_Util_Filter
             foreach (self::getIterator($directory, $suffix) as $file) {
                 self::addFileToFilter($file->getPathName(), $group);
             }
+        } else {
+            throw new RuntimeException($directory . ' does not exist');
         }
     }
 
@@ -164,8 +125,7 @@ class PHPUnit_Util_Filter
      *
      * @param  string $filename
      * @param  string $group
-     * @access public
-     * @static
+     * @throws RuntimeException
      * @since  Method available since Release 2.1.0
      */
     public static function addFileToFilter($filename, $group = 'DEFAULT')
@@ -180,6 +140,8 @@ class PHPUnit_Util_Filter
             else if (!in_array($filename, self::$blacklistedFiles[$group])) {
                 self::$blacklistedFiles[$group][] = $filename;
             }
+        } else {
+            throw new RuntimeException($filename . ' does not exist');
         }
     }
 
@@ -189,8 +151,7 @@ class PHPUnit_Util_Filter
      * @param  string $directory
      * @param  string $suffix
      * @param  string $group
-     * @access public
-     * @static
+     * @throws RuntimeException
      * @since  Method available since Release 3.1.5
      */
     public static function removeDirectoryFromFilter($directory, $suffix = '.php', $group = 'DEFAULT')
@@ -199,6 +160,8 @@ class PHPUnit_Util_Filter
             foreach (self::getIterator($directory, $suffix) as $file) {
                 self::removeFileFromFilter($file->getPathName(), $group);
             }
+        } else {
+            throw new RuntimeException($directory . ' does not exist');
         }
     }
 
@@ -207,8 +170,7 @@ class PHPUnit_Util_Filter
      *
      * @param  string $filename
      * @param  string $group
-     * @access public
-     * @static
+     * @throws RuntimeException
      * @since  Method available since Release 2.1.0
      */
     public static function removeFileFromFilter($filename, $group = 'DEFAULT')
@@ -223,6 +185,8 @@ class PHPUnit_Util_Filter
                     }
                 }
             }
+        } else {
+            throw new RuntimeException($filename . ' does not exist');
         }
     }
 
@@ -231,8 +195,7 @@ class PHPUnit_Util_Filter
      *
      * @param  string $directory
      * @param  string $suffix
-     * @access public
-     * @static
+     * @throws RuntimeException
      * @since  Method available since Release 3.1.5
      */
     public static function addDirectoryToWhitelist($directory, $suffix = '.php')
@@ -241,6 +204,8 @@ class PHPUnit_Util_Filter
             foreach (self::getIterator($directory, $suffix) as $file) {
                 self::addFileToWhitelist($file->getPathName());
             }
+        } else {
+            throw new RuntimeException($directory . ' does not exist');
         }
     }
 
@@ -251,8 +216,7 @@ class PHPUnit_Util_Filter
      * When the whitelist is not empty, whitelisting is used.
      *
      * @param  string $filename
-     * @access public
-     * @static
+     * @throws RuntimeException
      * @since  Method available since Release 3.1.0
      */
     public static function addFileToWhitelist($filename)
@@ -263,6 +227,8 @@ class PHPUnit_Util_Filter
             if (!in_array($filename, self::$whitelistedFiles)) {
                 self::$whitelistedFiles[] = $filename;
             }
+        } else {
+            throw new RuntimeException($filename . ' does not exist');
         }
     }
 
@@ -271,8 +237,7 @@ class PHPUnit_Util_Filter
      *
      * @param  string $directory
      * @param  string $suffix
-     * @access public
-     * @static
+     * @throws RuntimeException
      * @since  Method available since Release 3.1.5
      */
     public static function removeDirectoryFromWhitelist($directory, $suffix = '.php')
@@ -281,6 +246,8 @@ class PHPUnit_Util_Filter
             foreach (self::getIterator($directory, $suffix) as $file) {
                 self::removeFileFromWhitelist($file->getPathName());
             }
+        } else {
+            throw new RuntimeException($directory . ' does not exist');
         }
     }
 
@@ -288,8 +255,7 @@ class PHPUnit_Util_Filter
      * Removes a file from the whitelist.
      *
      * @param  string $filename
-     * @access public
-     * @static
+     * @throws RuntimeException
      * @since  Method available since Release 3.1.0
      */
     public static function removeFileFromWhitelist($filename)
@@ -302,6 +268,8 @@ class PHPUnit_Util_Filter
                     unset(self::$whitelistedFiles[$key]);
                 }
             }
+        } else {
+            throw new RuntimeException($filename . ' does not exist');
         }
     }
 
@@ -309,51 +277,47 @@ class PHPUnit_Util_Filter
      * Returns data about files within code coverage information, specifically
      * which ones will be filtered out and which ones may be whitelisted but not
      * touched by coverage.
-     * 
-     * Returns a two-item array. The first item is an array indexed by filenames 
+     *
+     * Returns a two-item array. The first item is an array indexed by filenames
      * with a boolean payload of whether they should be filtered out.
-     * 
-     * The second item is an array of filenames which are 
+     *
+     * The second item is an array of filenames which are
      * whitelisted but which are absent from the coverage information.
      *
      * @param  array   $codeCoverageInformation
      * @param  boolean $filterTests
      * @return array
-     * @access public
-     * @static
      */
     public static function getFileCodeCoverageDisposition(array $codeCoverageInformation, $filterTests = TRUE)
     {
         if (!self::$filter) {
             return array(array(), array());
-        }             
+        }
 
         $isFilteredCache = array();
         $coveredFiles    = array();
 
         foreach ($codeCoverageInformation as $k => $test) {
-            foreach (array_keys($test['files']) as $file) {
+            foreach (array_keys($test['executable']) as $file) {
                 if (!isset($isFilteredCache[$file])) {
                     $isFilteredCache[$file] = self::isFiltered(
                       $file, $filterTests
                     );
                 }
             }
-        }        
+        }
 
         $coveredFiles = array_keys($isFilteredCache);
-        $missedFiles  = array_diff(self::$whitelistedFiles,$coveredFiles);                
-        $missedFiles  = array_filter($missedFiles,'file_exists');
+        $missedFiles  = array_diff(self::$whitelistedFiles, $coveredFiles);
+        $missedFiles  = array_filter($missedFiles, 'file_exists');
 
-        return array($isFilteredCache,$missedFiles);
+        return array($isFilteredCache, $missedFiles);
     }
-    
+
     /**
      * @param  array   $codeCoverageInformation
-     * @param  boolean $addUncoveredFilesFromWhitelist
+     * @param  boolean $filterTests
      * @return array
-     * @access public
-     * @static
      */
     public static function getFilteredCodeCoverage(array $codeCoverageInformation, $filterTests = TRUE)
     {
@@ -368,28 +332,50 @@ class PHPUnit_Util_Filter
                         unset($codeCoverageInformation[$k]['files'][$file]);
                     }
                 }
+
+                foreach (array_keys($test['dead']) as $file) {
+                    if ($isFilteredCache[$file]) {
+                        unset($codeCoverageInformation[$k]['dead'][$file]);
+                    }
+                }
+
+                foreach (array_keys($test['executable']) as $file) {
+                    if ($isFilteredCache[$file]) {
+                        unset($codeCoverageInformation[$k]['executable'][$file]);
+                    }
+                }
             }
 
             if (self::$addUncoveredFilesFromWhitelist) {
-                foreach ($missedFiles as $missedFile) {
-                    xdebug_start_code_coverage(XDEBUG_CC_UNUSED | XDEBUG_CC_DEAD_CODE);
-                    include_once $missedFile;
-                    $coverage = xdebug_get_code_coverage();
-                    xdebug_stop_code_coverage();
+                foreach (self::$whitelistedFiles as $whitelistedFile) {
+                    if (!isset(self::$coveredFiles[$whitelistedFile])) {
+                        if (file_exists($whitelistedFile)) {
+                            xdebug_start_code_coverage(XDEBUG_CC_UNUSED | XDEBUG_CC_DEAD_CODE);
+                            include_once $whitelistedFile;
+                            $coverage = xdebug_get_code_coverage();
+                            xdebug_stop_code_coverage();
 
-                    if (isset($coverage[$missedFile])) {
-                        foreach ($coverage[$missedFile] as $line => $flag) {
-                            if ($flag > 0) {
-                                $coverage[$missedFile][$line] = -1;
+                            foreach ($coverage as $file => $fileCoverage)
+                            {
+                                if (!in_array($file, self::$whitelistedFiles) || isset(self::$coveredFiles[$file]))
+                                    continue;
+
+                                foreach ($fileCoverage as $line => $flag) {
+                                    if ($flag > 0) {
+                                        $fileCoverage[$line] = -1;
+                                    }
+                                }
+
+                                $codeCoverageInformation[] = array(
+                                  'test'  => NULL,
+                                  'files' => array(
+                                    $file => $fileCoverage
+                                  )
+                                );
+
+                                self::addCoveredFile($file);
                             }
                         }
-
-                        $codeCoverageInformation[] = array(
-                          'test'  => NULL,
-                          'files' => array(
-                            $missedFile => $coverage[$missedFile]
-                          )
-                        );
                     }
                 }
             }
@@ -405,8 +391,6 @@ class PHPUnit_Util_Filter
      * @param  boolean   $filterTests
      * @param  boolean   $asString
      * @return string
-     * @access public
-     * @static
      */
     public static function getFilteredStacktrace(Exception $e, $filterTests = TRUE, $asString = TRUE)
     {
@@ -416,7 +400,15 @@ class PHPUnit_Util_Filter
             $filteredStacktrace = array();
         }
 
-        foreach ($e->getTrace() as $frame) {
+        $eTrace = $e->getTrace();
+
+        if (!self::frameExists($eTrace, $e->getFile(), $e->getLine())) {
+            array_unshift(
+              $eTrace, array('file' => $e->getFile(), 'line' => $e->getLine())
+            );
+        }
+
+        foreach ($eTrace as $frame) {
             if (!self::$filter || (isset($frame['file']) && !self::isFiltered($frame['file'], $filterTests, TRUE))) {
                 if ($asString === TRUE) {
                     $filteredStacktrace .= sprintf(
@@ -439,8 +431,6 @@ class PHPUnit_Util_Filter
      *
      * @param  boolean $filter
      * @throws InvalidArgumentException
-     * @access public
-     * @static
      * @since  Method available since Release 3.0.0
      */
     public static function setFilter($filter)
@@ -460,8 +450,6 @@ class PHPUnit_Util_Filter
      * @param  string $directory
      * @param  string $suffix
      * @return Iterator
-     * @access protected
-     * @static
      * @since  Method available since Release 3.1.5
      */
     protected static function getIterator($directory, $suffix)
@@ -479,33 +467,15 @@ class PHPUnit_Util_Filter
      * @param  boolean $filterTests
      * @param  boolean $ignoreWhitelist
      * @return boolean
-     * @access protected
-     * @static
      * @since  Method available since Release 2.1.3
      */
-    protected static function isFiltered($filename, $filterTests = TRUE, $ignoreWhitelist = FALSE)
+    public static function isFiltered($filename, $filterTests = TRUE, $ignoreWhitelist = FALSE)
     {
         $filename = realpath($filename);
 
         // Use blacklist.
         if ($ignoreWhitelist || empty(self::$whitelistedFiles)) {
-            if (DIRECTORY_SEPARATOR == '\\' &&
-                !self::$blackListConverstionForWindowsDone) {
-                $count = count(self::$blacklistedFiles['PEAR']);
-
-                for ($i = 0; $i < $count; $i++) {
-                    self::$blacklistedFiles['PEAR'][$i] = str_replace(
-                      '/', '\\', self::$blacklistedFiles['PEAR'][$i]
-                    );
-                }
-
-                self::$blackListConverstionForWindowsDone = TRUE;
-            }
-
-            $blacklistedFiles = array_merge(
-              self::$blacklistedFiles['DEFAULT'],
-              self::$blacklistedFiles['PEAR']
-            );
+            $blacklistedFiles = self::$blacklistedFiles['DEFAULT'];
 
             if ($filterTests) {
                 $blacklistedFiles = array_merge(
@@ -544,6 +514,47 @@ class PHPUnit_Util_Filter
             return TRUE;
         }
     }
+
+    /**
+     * Adds a file to the list of covered files.
+     *
+     * @param  string  $filename
+     * @since  Method available since Release 3.3.0
+     */
+    public static function addCoveredFile($filename)
+    {
+        self::$coveredFiles[$filename] = TRUE;
+    }
+
+    /**
+     * Returns the list of covered files.
+     *
+     * @return array
+     * @since  Method available since Release 3.3.0
+     */
+    public static function getCoveredFiles()
+    {
+        return self::$coveredFiles;
+    }
+
+    /**
+     * @param  array  $trace
+     * @param  string $file
+     * @param  int    $line
+     * @return boolean
+     * @since  Method available since Release 3.3.2
+     */
+    public static function frameExists(array $trace, $file, $line)
+    {
+        foreach ($trace as $frame) {
+            if (isset($frame['file']) && $frame['file'] == $file &&
+                isset($frame['line']) && $frame['line'] == $line) {
+                return TRUE;
+            }
+        }
+        
+        return FALSE;
+    }
 }
 
 PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');
