@@ -2,7 +2,7 @@
 /**
  * PHPUnit
  *
- * Copyright (c) 2002-2008, Sebastian Bergmann <sb@sebastian-bergmann.de>.
+ * Copyright (c) 2002-2009, Sebastian Bergmann <sb@sebastian-bergmann.de>.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,9 +37,9 @@
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    SVN: $Id: Printer.php 2141 2008-01-17 10:49:39Z sb $
+ * @version    SVN: $Id: Printer.php 4404 2008-12-31 09:27:18Z sb $
  * @link       http://www.phpunit.de/
  * @since      File available since Release 2.0.0
  */
@@ -54,30 +54,34 @@ PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');
  * @category   Testing
  * @package    PHPUnit
  * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
+ * @copyright  2002-2009 Sebastian Bergmann <sb@sebastian-bergmann.de>
  * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    Release: 3.2.16
+ * @version    Release: 3.3.16
  * @link       http://www.phpunit.de/
  * @since      Class available since Release 2.0.0
  * @abstract
  */
 abstract class PHPUnit_Util_Printer
 {
+    /**
+     * If TRUE, flush output after every write.
+     *
+     * @var boolean
+     */
+    protected $autoFlush = FALSE;
+
     /**
      * @var    resource
-     * @access protected
      */
     protected $out;
 
     /**
      * @var    string
-     * @access protected
      */
     protected $outTarget;
 
     /**
      * @var    boolean
-     * @access protected
      */
     protected $printsHTML = FALSE;
 
@@ -86,7 +90,6 @@ abstract class PHPUnit_Util_Printer
      *
      * @param  mixed $out
      * @throws InvalidArgumentException
-     * @access public
      */
     public function __construct($out = NULL)
     {
@@ -97,7 +100,7 @@ abstract class PHPUnit_Util_Printer
 
                     if (sizeof($out) != 2) {
                         throw new InvalidArgumentException;
-                    }                    
+                    }
 
                     $this->out = fsockopen($out[0], $out[1]);
                 } else {
@@ -114,7 +117,6 @@ abstract class PHPUnit_Util_Printer
     /**
      * Flush buffer, optionally tidy up HTML, and close output.
      *
-     * @access public
      */
     public function flush()
     {
@@ -129,20 +131,74 @@ abstract class PHPUnit_Util_Printer
         }
     }
 
+    /**
+     * Performs a safe, incremental flush.
+     *
+     * Do not confuse this function with the flush() function of this class,
+     * since the flush() function may close the file being written to, rendering
+     * the current object no longer usable.
+     *
+     * @since  Method available since Release 3.3.0
+     */
+    public function incrementalFlush()
+    {
+        if ($this->out !== NULL) {
+            fflush($this->out);
+        } else {
+            flush();
+        }
+    }
+
     /**
      * @param  string $buffer
-     * @access public
      */
     public function write($buffer)
     {
         if ($this->out !== NULL) {
             fwrite($this->out, $buffer);
+
+            if ($this->autoFlush) {
+                $this->incrementalFlush();
+            }
         } else {
-            if (php_sapi_name() != 'cli') {
-                $buffer = htmlentities($buffer);
+            if (PHP_SAPI != 'cli') {
+                $buffer = htmlspecialchars($buffer);
             }
 
             print $buffer;
+
+            if ($this->autoFlush) {
+                $this->incrementalFlush();
+            }
+        }
+    }
+
+    /**
+     * Check auto-flush mode.
+     *
+     * @return boolean
+     * @since  Method available since Release 3.3.0
+     */
+    public function getAutoFlush()
+    {
+        return $this->autoFlush;
+    }
+
+    /**
+     * Set auto-flushing mode.
+     *
+     * If set, *incremental* flushes will be done after each write. This should
+     * not be confused with the different effects of this class' flush() method.
+     *
+     * @param boolean $autoFlush
+     * @since  Method available since Release 3.3.0
+     */
+    public function setAutoFlush($autoFlush)
+    {
+        if (is_bool($autoFlush)) {
+            $this->autoFlush = $autoFlush;
+        } else {
+            throw new InvalidArgumentException;
         }
     }
 }
